/*
 *
 * ATM Switch Control Library - swctl
 * 	by miwa@mt.cs.keio.ac.jp, 1997
 * $Id: swctlupc.c,v 1.1.4.1 1998/09/02 04:02:31 onoe Exp $
 *
 */

#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <ctype.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <ucd-snmp/ucd-snmp-config.h>
#include <ucd-snmp/ucd-snmp-includes.h>

#include "config.h"

#include "swctl.h"
#include "swctl_snmp.h"

/* search upc_index from param */
int
upc_search_param2index(swctl_upc_table_t *upc_table_list,
		       swctl_param_t *param, int *upc_index)
{
    swctl_upc_table_t	*upc_table;

    if( !param ) {		/* param is not defined */
	*(upc_index) = 0;	/* return: upc_index = 0 */
	return(1);
    }
	  
    for( upc_table = upc_table_list; upc_table; upc_table =
	upc_table->next_table) {
	if( !memcmp((void *)upc_table->param, (void *)param,
		   sizeof(swctl_param_t))) {
	    *(upc_index) = upc_table->index;	/* match */
	    return(1);
	}
    }

    /* no existent; new upc_index = max_upc_index + 1 */
    *(upc_index) = upc_table_list->index + 1;

    return(0);
}

/* search upc_table from index */
swctl_upc_table_t *
upc_search_index2table(swctl_upc_table_t *upc_table_list, int upc_index)
{
    swctl_upc_table_t	*upc_table;

    for( upc_table = upc_table_list; upc_table; upc_table =
	upc_table->next_table)
      if( upc_table->index == upc_index )
	return(upc_table);

    /* upc index is no existent */
    return((swctl_upc_table_t *)NULL);
}

swctl_upc_table_t *
upc_index_entry(swctl_upc_table_t *upc_table_list, int upc_index,
		swctl_param_t *param)
{
    swctl_upc_table_t	*upc_table;

    if( !param )
      return(upc_table_list);

    upc_table = (swctl_upc_table_t
			     *)malloc(sizeof(swctl_upc_table_t));

    upc_table->next_table = upc_table_list;

    upc_table->status = SNMP_VALID_VAL;
    upc_table->index = upc_index;
    upc_table->param = (swctl_param_t *)malloc(sizeof(swctl_param_t));
    memcpy((void *)upc_table->param, (void *)param, sizeof(swctl_param_t));

    return(upc_table);
}

swctl_upc_table_t *
upc_index_delete(swctl_upc_table_t *upc_table_list, int upc_index)
{
    swctl_upc_table_t	*current;
    swctl_upc_table_t	*previous;
    swctl_upc_table_t	*return_list;

    return_list = (swctl_upc_table_t *)NULL;

    for( current = upc_table_list,
	previous = (swctl_upc_table_t *)NULL;
	current;
	previous = current, current = current->next_table ) {
	if( current->index == upc_index ) {

	    if( previous == (swctl_upc_table_t *)NULL ) {
		/* delete upc_table_list head entry */
		return_list = current->next_table;
	    }
	    else {
		/* delete upc_table_list mid or tail entry */
		previous->next_table = current->next_table;
		return_list = upc_table_list;
	    }
	    if(current->param)
	      free(current->param);
	    free(current);
	    break;
	}
    }

    return(return_list);
}

swctl_mib_object_t *
make_upc_object(miboid *prefix, int prefix_len, int upc_index, int value)
{
    miboid suffix[1];
    swctl_mib_object_t *mib_object;

    suffix[0] = upc_index;

    mib_object = make_mib_object(prefix, prefix_len, suffix, 1, value);

    return(mib_object);
}

swctl_mib_object_t *
make_upc_object_list(int upc_index, swctl_param_t *param)
{
    swctl_mib_object_t *mib_object_list = (swctl_mib_object_t *)NULL;
    swctl_mib_object_t *mib_object;

    if( upc_index == 0 || !param )	/* upc param is not defined */
      return((swctl_mib_object_t *)NULL);


    if(param->pcr01 > 0) {

	mib_object = make_upc_object(SNMP_UPC_PCR01_PREFIX,
				     SNMP_UPC_PREFIX_LENGTH,
				     upc_index, param->pcr01);
	mib_object->next_object = mib_object_list;
	mib_object_list = mib_object;
    }

    if(param->scr01 > 0) {

	mib_object = make_upc_object(SNMP_UPC_SCR01_PREFIX,
				     SNMP_UPC_PREFIX_LENGTH,
				     upc_index, param->scr01);
	mib_object->next_object = mib_object_list;
	mib_object_list = mib_object;
    }

    if(param->mbs01 > 0) {
	
	mib_object = make_upc_object(SNMP_UPC_MBS01_PREFIX,
				     SNMP_UPC_PREFIX_LENGTH,
				     upc_index, param->mbs01);
	mib_object->next_object = mib_object_list;
	mib_object_list = mib_object;
    }

    if(param->pcr0 > 0) {

	mib_object = make_upc_object(SNMP_UPC_PCR0_PREFIX,
				     SNMP_UPC_PREFIX_LENGTH,
				     upc_index, param->pcr0);
	mib_object->next_object = mib_object_list;
	mib_object_list = mib_object;
    }

    if(param->scr0 > 0) {

	mib_object = make_upc_object(SNMP_UPC_SCR0_PREFIX,
				     SNMP_UPC_PREFIX_LENGTH,
				     upc_index, param->scr0);
	mib_object->next_object = mib_object_list;
	mib_object_list = mib_object;
    }

    if(param->mbs0 > 0) {

	mib_object = make_upc_object(SNMP_UPC_MBS0_PREFIX,
				     SNMP_UPC_PREFIX_LENGTH,
				     upc_index, param->mbs0);
	mib_object->next_object = mib_object_list;
	mib_object_list = mib_object;
    }

    if(param->cdvt > 0) {

	mib_object = make_upc_object(SNMP_UPC_CDVT_PREFIX,
				     SNMP_UPC_PREFIX_LENGTH,
				     upc_index, param->cdvt);
	mib_object->next_object = mib_object_list;
	mib_object_list = mib_object;
    }
	  
    if(param->act > 0) {

	mib_object = make_upc_object(SNMP_UPC_ACT_PREFIX,
				     SNMP_UPC_PREFIX_LENGTH,
				     upc_index, param->act);
	mib_object->next_object = mib_object_list;
	mib_object_list = mib_object;
    }

    if(param->aal5epd > 0) {

	mib_object = make_upc_object(SNMP_UPC_AAL5EPD_PREFIX,
				     SNMP_UPC_PREFIX_LENGTH,
				     upc_index, param->aal5epd);
	mib_object->next_object = mib_object_list;
	mib_object_list = mib_object;
    }

    return(mib_object_list);
}

int
print_upc_table(swctl_upc_table_t *upc_table)
{

    if(upc_table) {

	fprintf(stdout, "%d\t", upc_table->index);
	fprintf(stdout, "%u\t", upc_table->param->pcr01);
	fprintf(stdout, "%u\t", upc_table->param->scr01);
	fprintf(stdout, "%u\t", upc_table->param->mbs01);
	fprintf(stdout, "%u\t", upc_table->param->pcr0);
	fprintf(stdout, "%u\t", upc_table->param->scr0);
	fprintf(stdout, "%u\t", upc_table->param->mbs0);
	fprintf(stdout, "%u\t", upc_table->param->cdvt);
	if(upc_table->param->act == 0) 
	  fprintf(stdout, "drop\t");
	else
	  fprintf(stdout, "tag\t");
	if(upc_table->param->aal5epd == 0) 
	  fprintf(stdout, "no\n");
	else
	  fprintf(stdout, "yes\n");
    }    
    return(0);
}

int
print_upc_table_list(swctl_upc_table_t *upc_table_list)
{
    swctl_upc_table_t *upc_table;

    fprintf(stdout,
	    "index\tPCR01\tSCR01\tMBS01\tPCR0\tSCR0\tMBS0\tCDVT\tACT\tAAL5EPD\n");

    for( upc_table = upc_table_list; upc_table; upc_table =
	upc_table->next_table )
      print_upc_table(upc_table);

    return(0);
}    
	
int
free_upc_table_list(swctl_upc_table_t *upc_table_list)
{
    swctl_upc_table_t *current;
    swctl_upc_table_t *previous;

    for( current = upc_table_list, previous = (swctl_upc_table_t *)NULL;
	current;
	previous = current, current = current->next_table,
	free(previous) ) {
	if(current->param)
	  free(current->param);
    }

    return(0);
}
